*==============================================================================
* Program:			CreateCHM.prg
* Purpose:			Creates a CHM files from a set of HTML documents and a
*						driver table
* Author:			Doug Hennig
* Last revision:	03/05/2002
* Parameters:		tcCHMFile  - the name (including path) of the CHM file to
*						create (optional: if it isn't passed, CHM.CHM in the
*						HTMLHelp subdirectory of the current directory is
*						assumed)
*					tcCSSFile  - the name of the CSS file to use (optional: if
*						it isn't passed, no CSS file is used)
*					tcTitle    - the title for the CHM file (optional: if it
*						isn't passed, "No title specified" is used)
* Returns:			.T. if everything worked
* Environment in:	all files needed for the CHM file are in the same directory
*						as the CHM file to be created
*					CONTENTS.DBF is available to be opened
* Environment out:	the CHM file has been created
*					if any problems occurred, they're logged in a log file,
*						LOG.TXT, which is automatically displayed at the end
*==============================================================================

lparameters tcCHMFile, ;
	tcCSSFile, ;
	tcTitle
local lcCHMFile, ;
	lcTitle, ;
	lcCSSFile, ;
	lcHHPFile, ;
	lcHFile, ;
	lcDefault, ;
	lcFile, ;
	lnSection, ;
	lnLevel, ;
	lcPrefix0, ;
	lcPrefix1, ;
	lcPrefix2, ;
	lnTopics, ;
	laTopics[1], ;
	lnI, ;
	lcKeyword, ;
	loRegistry, ;
	lcPath, ;
	llGotPath
#include Assemble.h

* Ensure the proper parameters were passed.

lcCHMFile = iif(vartype(tcCHMFile) = 'C' and not empty(tcCHMFile), ;
	tcCHMFile, 'HTMLHelp\CHM.CHM')
lcTitle   = iif(vartype(tcTitle) = 'C' and not empty(tcTitle), ;
	tcTitle, 'No title specified')
lcCSSFile = iif(vartype(tcCSSFile) = 'C', tcCSSFile, '')
if not empty(lcCSSFile) and not file(lcCSSFile)
	messagebox('CSS file ' + lcCSSFile + ' not found.')
	return .F.
endif not empty(lcCSSFile) ...
lcHHPFile = forceext(lcCHMFile, 'hhp')
lcHFile   = forceext(lcCHMFile, 'h')

* Erase the CHM file if it exists.

erase (lcCHMFile)

* Open the contents table.

close tables all
use CONTENTS order MAIN again shared in 0

* Find the default page.

locate for DEFAULT
lcDefault = alltrim(FILENAME)

* Generate the HHP (project) file. First, the fixed part.

set textmerge on to (lcHHPFile) noshow
\\[OPTIONS]
\Compatibility=1.1
\Compiled file=<<justfname(lcCHMFile)>>
\Contents file=<<justfname(forceext(lcCHMFile, 'hhc'))>>
\Auto index=Yes
\Default topic=<<lcDefault>>
\Display compile progress=No
\Full-text search=Yes
\Index file=<<justfname(forceext(lcCHMFile, 'hhk'))>>
\Language=0x409 English (United States)
\Title=<<lcTitle>>
\Default Window=HelpWindow
\
\[WINDOWS]
\HelpWindow=,"<<justfname(forceext(lcCHMFile, 'hhc'))>>","<<justfname(forceext(lcCHMFile, 'hhk'))>>","<<lcDefault>>","<<lcDefault>>",,,,,0x2520,,0x387e,,,,,0,,,
\
\[FILES]
if not empty(lcCSSFile)
	\<<justfname(lcCSSFile)>>
endif not empty(lcCSSFile)

* Output the list of unique file names.

select FILENAME from CONTENTS into cursor FILES group by 1
scan
	\<<trim(FILENAME)>>
endscan
use
select CONTENTS

* Output the help IDs and map section.

\
\[ALIAS]
scan for INTOC
	lcFile = trim(FILENAME)
	\IDH_<<upper(juststem(lcFile)))>>=<<lcFile>>
endscan for INTOC
\
\[MAP]
\#include <<justfname(lcHFile)>>

* Write out the file.

\
set textmerge to

* Generate the include file.

set textmerge on to (lcHFile) noshow
scan for INTOC
	lcFile = trim(FILENAME)
	\\#define IDH_<<upper(juststem(lcFile)))>> <<HELPID>>
	\
endscan for INTOC
set textmerge to

* Generate the HHC (contents) file. First, header information.

set textmerge on to (forceext(lcCHMFile, 'hhc')) noshow
\\<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">
\<HTML>
\<HEAD>
\<meta name="GENERATOR" content="Stonefield Systems Group Inc.">
\<!-- Sitemap 1.0 -->
\</HEAD>
\<BODY>
\<UL>

* Next, the topics.

lnSection = -1
lnLevel   = 0
scan for INTOC

* Determine the indentation.

	lcPrefix0 = replicate(ccTAB, LEVEL)
	lcPrefix1 = replicate(ccTAB, LEVEL + 1)
	lcPrefix2 = lcPrefix1 + ccTAB

* If we're on a different section or a lower level, insert a </UL> tag.

	do case
		case SECTION <> lnSection and lnSection > 0
			\<<ccTAB>></UL>
		case LEVEL < lnLevel
			\<<lcPrefix1>></UL>

* If we're at a higher level than before, insert a <UL> tag.

		case LEVEL > lnLevel
			\<<lcPrefix0>><UL>
	endcase
	lnSection = SECTION
	lnLevel   = LEVEL

* Create the information for the current file.

	\<<lcPrefix1>><LI><OBJECT type="text/sitemap">
	\<<lcPrefix2>><param name="Name" value="<<ConvertTitle(NAME)>>">
	\<<lcPrefix2>><param name="Local" value="<<trim(FILENAME)>>">
	if not empty(IMAGE)
		\<<lcPrefix2>><param name="ImageNumber" value="<<transform(IMAGE)>>">
	endif not empty(IMAGE)
	\<<lcPrefix1>></OBJECT>
endscan for INTOC

* Finish off the file and write it out.

\<<ccTAB>></UL>
\</UL>
\</BODY>
\</HTML>
\
set textmerge to

* Generate the HHK (index) file.

set textmerge on to (forceext(lcCHMFile, 'hhk')) noshow
\\<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">
\<HTML>
\<HEAD>
\<meta name="GENERATOR" content="Stonefield Systems Group Inc.">
\<!-- Sitemap 1.0 -->
\</HEAD>
\<BODY>
\<UL>

* Now, the topics. The "param name="Name" statement is used twice because HTML
* Help Workshop requires it.

scan for ININDEX
	lcFile = trim(FILENAME)
	if empty(INDEX)
		lnTopics = 1
		dimension laTopics[1]
		laTopics[1] = trim(NAME)
	else
		lnTopics = alines(laTopics, INDEX)
	endif empty(INDEX)
	lcPrefix1 = ccTAB
	lcPrefix2 = ccTAB + ccTAB
	for lnI = 1 to lnTopics
		lcKeyword = ConvertTitle(laTopics[lnI])
		\<<lcPrefix1>><LI><OBJECT type="text/sitemap">
		\<<lcPrefix2>><param name="Name" value="<<lcKeyword>>">
		\<<lcPrefix2>><param name="Name" value="<<lcKeyword>>">
		\<<lcPrefix2>><param name="Local" value="<<lcFile>>">
		\<<lcPrefix1>></OBJECT>
	next lnI
endscan for ININDEX

* Finish off the file and write it out.

\</UL>
\</BODY>
\</HTML>
\
set textmerge to

* Find the location of HTML Help Workshop and compile the CHM file if we found
* it.

loRegistry = newobject('Registry', home() + 'FFC\Registry.vcx')
lcPath     = ''
llGotPath  = loRegistry.GetRegKey('Path', @lcPath, ;
	'Software\Microsoft\Windows\CurrentVersion\App Paths\hhw.exe', ;
	HKEY_LOCAL_MACHINE) = 0
if llGotPath
	lcPath = ShortPath(forcepath('hhc.exe', lcPath))
	erase log.txt
	run &lcPath &lcHHPFile > log.txt
	if file('log.txt')
		modify file log.txt nowait
	else
		erase log.txt
	endif file('log.txt')
else
	messagebox('Cannot locate HTML Help Workshop')
endif llGotPath

* Display the CHM file if it was created.

if file(lcCHMFile)
	declare integer ShellExecute in SHELL32.DLL integer nWinHandle, ;
		string cOperation, string cFileName, string cParameters, ;
		string cDirectory, integer nShowWindow
	ShellExecute(0, 'Open', lcCHMFile, '', '', 1)
endif file(lcCHMFile)

* Convert illegal characters in topic titles.

function ConvertTitle(tcTitle)
local lcTitle
lcTitle = strtran(trim(tcTitle), '&', '&amp;')
lcTitle = strtran(lcTitle, '"', '&quot;')
lcTitle = strtran(lcTitle, chr(151), '--')
return lcTitle

* Get the short (8.3) path for the specified path.

function ShortPath(tcPath)
local lcPath, ;
	lnLength, ;
	lcBuffer, ;
	lnResult
declare integer GetShortPathName in Win32API ;
	string @lpszLongPath, string @lpszShortPath, integer cchBuffer
lcPath   = tcPath
lnLength = 260
lcBuffer = space(lnLength)
lnResult = GetShortPathName(@lcPath, @lcBuffer, lnLength)
return iif(lnResult = 0, '', left(lcBuffer, lnResult))
