*==============================================================================
* Program:			Process.prg
* Purpose:			Create HTML files ready to compile into a CHM file from a
*						set of Word documents
* Author:			Doug Hennig
* Last revision:	02/17/2001
* Parameters:		tcWordDocs - where to put Word documents to process
*						(optional: if it isn't passed, the WordDocs
*						subdirectory of the current directory is assumed)
*					tcHTMLDir  - where to put HTML files generated by Word
*						(optional: if it isn't passed, the HTMLDocs
*						subdirectory of the current directory is assumed)
*					tcCHMDir   - where to put processed HTML files and the CHM
*						file (optional: if it isn't passed, the HTMLHelp
*						subdirectory of the current directory is assumed)
*					tcCSSFile  - the name of the CSS file to use (optional: if
*						it isn't passed, no CSS file is used)
*					tcLogFile  - the file to log problems to (optional: if it
*						isn't passed, "CHMLog.log" in the current directory is
*						used)
*					tlNoGenerateHTML - .T. to not generate HTML from the Word
*						documents (this may have been done previously)
* Returns:			.T. if everything worked
* Environment in:	the Word documents exist in tcWordDocs
* Environment out:	if they didn't exist, the directories specified in
*						tcHTMLDir and tcCHMDir are created
*					HTML files have been created from Word documents and
*						cleaned up and processed extensively
*					if any problems occurred, they're logged in the log file,
*						which is automatically displayed at the end
* Notes:			This process uses the following tables:
*						SAR: the list of search-and-replace items
*						PROCESSES: the list of processes to apply to the HTML
*							files
*==============================================================================

lparameters tcWordDocs, ;
	tcHTMLDir, ;
	tcCHMDir, ;
	tcCSSFile, ;
	tcLogFile, ;
	tlNoGenerateHTML
local lcWordDocs, ;
	lcHTMLDir, ;
	lcCHMDir, ;
	lcCSSFile, ;
	lcLogFile, ;
	lcRunDir, ;
	lcCurrPath, ;
	loIterator, ;
	loProcess
#include Assemble.H

* Handle the directories, CSS file, and log file not passed by assigning
* defaults. Ensure all directories exist (create all but the WordDocs directory
* if not).

lcWordDocs = iif(vartype(tcWordDocs) = 'C' and not empty(tcWordDocs), ;
	tcWordDocs, 'WordDocs')
if empty(lcWordDocs) or not directory(lcWordDocs)
	messagebox('Directory ' + lcWordDocs + ' not found.')
	return .F.
endif empty(lcWordDocs) ...
lcHTMLDir = iif(vartype(tcHTMLDir) = 'C' and not empty(tcHTMLDir), tcHTMLDir, ;
	'HTMLDocs')
if not directory(lcHTMLDir)
	md (lcHTMLDir)
	if not directory(lcHTMLDir)
		messagebox('Directory ' + lcHTMLDir + ' not found.')
		return .F.
	endif not directory(lcHTMLDir)
endif not directory(lcHTMLDir)
lcCHMDir = iif(vartype(tcCHMDir) = 'C' and not empty(tcCHMDir), tcCHMDir, ;
	'HTMLHelp')
if not directory(lcCHMDir)
	md (lcCHMDir)
	if not directory(lcCHMDir)
		messagebox('Directory ' + lcCHMDir + ' not found.')
		return .F.
	endif not directory(lcCHMDir)
endif not directory(lcCHMDir)
lcCSSFile = iif(vartype(tcCSSFile) = 'C', tcCSSFile, '')
if not empty(lcCSSFile) and not file(lcCSSFile)
	messagebox('CSS file ' + lcCSSFile + ' not found.')
	return .F.
endif not empty(lcCSSFile) ...
lcWordDocs = addbs(fullpath(lcWordDocs))
lcHTMLDir  = addbs(fullpath(lcHTMLDir))
lcCHMDir   = addbs(fullpath(lcCHMDir))
lcLogFile  = iif(vartype(tcLogFile) = 'C' and not empty(tcLogFile), ;
	tcLogFile, 'CHMLog.log')

* Set a few environmental things.

set escape on
set safety off
close databases
lcRunDir   = addbs(justpath(sys(16)))
lcCurrPath = set('PATH')
set path to &lcCurrPath., &lcRunDir
set classlib to ASSEMBLE additive

* Erase the log file so we start with a clean slate.

erase (lcLogFile)

* If we're supposed to, convert the Word documents to HTML. Use the
* GenerateHTML process object with the FileIterator driver. Graphic files are
* just copied.

if not tlNoGenerateHTML
	loIterator = createobject('FileIterator')
	with loIterator
		.oProcess   = createobject('GenerateHTML')
		.cDirectory = lcWordDocs
		.cWriteDir  = lcHTMLDir
		.cLogFile   = lcLogFile
		.GetFiles()
		.Process('Converting Word docs to HTML...')
	endwith
endif not tlNoGenerateHTML

* Clean up the HTML by stripping out all of Word's crap. We'll use a new
* FileIterator because we'll be processing a different directory. Fill the
* ReplaceText process object with the things to clean up that are defined in
* SAR.DBF.

loIterator = createobject('FileIterator')
with loIterator
	.cDirectory    = lcHTMLDir
	.cWriteDir     = lcCHMDir
	.cFileSkeleton = '*.htm*'
	.cLogFile      = lcLogFile
	.GetFiles()
endwith
loProcess = createobject('ReplaceText')
with loProcess
	use SAR order ORDER
	scan for ORDER >= 0
		.AddSearchAndReplace(textmerge(alltrim(SEARCHFOR)), ;
			textmerge(alltrim(REPLACE)))
	endscan for ORDER >= 0
	use
endwith

* Now do the process.

loIterator.oProcess = loProcess
loIterator.Process('Cleaning up HTML docs...')

* Copy the CSS and image files to the CHM directory.

if not empty(lcCSSFile)
	copy file (lcCSSFile) to forcepath(lcCSSFile, lcCHMDir)
endif not empty(lcCSSFile)
loIterator = createobject('FileIterator')
with loIterator
	.oProcess      = createobject('CopyFile')
	.cDirectory    = lcHTMLDir
	.cWriteDir     = lcCHMDir
	.cFileSkeleton = '*.*'
	.cLogFile      = lcLogFile
	.aExclude[1]   = 'html'
	.GetFiles()
endwith
loIterator.Process('Copying additional files...')

* Now process the HTML files in various ways, as defined in the PROCESSES
* table.

loIterator = createobject('FileIteratorMultipleProcess')
with loIterator
	.cDirectory    = lcCHMDir
	.cFileSkeleton = '*.htm*'
	.cLogFile      = lcLogFile
	.GetFiles()
	select 0
	use PROCESSES order ORDER
	count for ORDER >= 0 to lnProcesses
	dimension .aProcesses[lnProcesses]
	lnProcess = 0
	scan for ORDER >= 0
		lnProcess = lnProcess + 1
		loProcess = newobject(alltrim(CLASS), alltrim(LIBRARY))
		loProcess.cTitle = alltrim(TITLE)
		if not empty(CODE)
			execscript(CODE, loProcess)
		endif not empty(CODE)
		.aProcesses[lnProcess] = loProcess
	endscan for ORDER >= 0
	use
	.Process('Additional processing...')
endwith

* Now we'll check out any errors.

if file(lcLogFile)
	modify file (lcLogFile) nowait
endif file(lcLogFile)

* Clean up before we exit.

set path to &lcCurrPath
release classlib alias ASSEMBLE
